from __future__ import absolute_import, division, print_function

import json

import pytest

import stripe


class TestListObject(object):
    @pytest.fixture
    def list_object(self):
        return stripe.ListObject.construct_from({
            'object': 'list',
            'url': '/my/path',
            'data': ['foo'],
        }, 'mykey')

    def assert_response(self, res):
        assert isinstance(res[0], stripe.Charge)
        assert res[0].foo == 'bar'

    def test_for_loop(self, list_object):
        seen = []

        for item in list_object:
            seen.append(item)

        assert seen == ['foo']

    def test_list(self, request_mock, list_object):
        request_mock.stub_request(
            'get',
            '/my/path',
            [
                {
                    'object': 'charge',
                    'foo': 'bar',
                },
            ]
        )

        res = list_object.list(myparam='you')

        request_mock.assert_requested(
            'get',
            '/my/path',
            {
                'myparam': 'you',
            },
            None
        )
        self.assert_response(res)

    def test_create(self, request_mock, list_object):
        request_mock.stub_request(
            'post',
            '/my/path',
            [
                {
                    'object': 'charge',
                    'foo': 'bar',
                },
            ]
        )

        res = list_object.create(myparam='eter')

        request_mock.assert_requested(
            'post',
            '/my/path',
            {
                'myparam': 'eter',
            },
            None
        )
        self.assert_response(res)

    def test_retrieve(self, request_mock, list_object):
        request_mock.stub_request(
            'get',
            '/my/path/myid',
            [
                {
                    'object': 'charge',
                    'foo': 'bar',
                },
            ]
        )

        res = list_object.retrieve('myid', myparam='cow')

        request_mock.assert_requested(
            'get',
            '/my/path/myid',
            {
                'myparam': 'cow',
            },
            None
        )

        self.assert_response(res)

    def test_len(self, list_object):
        assert len(list_object) == 1

    def test_bool(self, list_object):
        assert list_object

        empty = stripe.ListObject.construct_from({
            'object': 'list',
            'url': '/my/path',
            'data': [],
        }, 'mykey')
        assert bool(empty) is False

    def test_serialize_empty_list(self):
        empty = stripe.ListObject.construct_from({
            'object': 'list',
            'data': [],
        }, 'mykey')
        serialized = str(empty)
        deserialized = stripe.ListObject.construct_from(
            json.loads(serialized), 'mykey')
        assert deserialized == empty

    def test_serialize_nested_empty_list(self):
        empty = stripe.ListObject.construct_from({
            'object': 'list',
            'data': [],
        }, 'mykey')
        obj = stripe.stripe_object.StripeObject.construct_from({
            'nested': empty,
        }, 'mykey')
        serialized = str(obj)
        deserialized = stripe.stripe_object.StripeObject.construct_from(
            json.loads(serialized), 'mykey')
        assert deserialized.nested == empty


class TestAutoPaging:

    @staticmethod
    def pageable_model_response(ids, has_more):
        return {
            'object': 'list',
            'url': '/v1/pageablemodels',
            'data': [{'id': id, 'object': 'pageablemodel'} for id in ids],
            'has_more': has_more,
        }

    def test_iter_one_page(self, request_mock):
        lo = stripe.ListObject.construct_from(
            self.pageable_model_response(['pm_123', 'pm_124'], False),
            'mykey'
        )

        request_mock.assert_no_request()

        seen = [item['id'] for item in lo.auto_paging_iter()]

        assert seen == ['pm_123', 'pm_124']

    def test_iter_two_pages(self, request_mock):
        lo = stripe.ListObject.construct_from(
            self.pageable_model_response(['pm_123', 'pm_124'], True),
            'mykey'
        )
        lo._retrieve_params = {'foo': 'bar'}

        request_mock.stub_request(
            'get',
            '/v1/pageablemodels',
            self.pageable_model_response(['pm_125', 'pm_126'], False)
        )

        seen = [item['id'] for item in lo.auto_paging_iter()]

        request_mock.assert_requested(
            'get',
            '/v1/pageablemodels',
            {
                'starting_after': 'pm_124',
                'foo': 'bar'
            },
            None
        )

        assert seen == ['pm_123', 'pm_124', 'pm_125', 'pm_126']

    def test_class_method_two_pages(self, request_mock):
        request_mock.stub_request(
            'get',
            '/v1/charges',
            {
                'object': 'list',
                'data': [{'id': 'ch_001'}],
                'url': '/v1/charges',
                'has_more': False,
            }
        )

        seen = [item['id'] for item in stripe.Charge.auto_paging_iter(
            limit=25,
            foo='bar'
        )]

        request_mock.assert_requested(
            'get',
            '/v1/charges',
            {
                'limit': 25,
                'foo': 'bar',
            }
        )
        assert seen == ['ch_001']



from __future__ import absolute_import, division, print_function

import sys
from collections import namedtuple

from stripe import util
from stripe.six.moves import builtins

PRINT_FUNC_STRING = builtins.__name__ + '.print'

LogTestCase = namedtuple('LogTestCase', 'env flag should_output')
FmtTestCase = namedtuple('FmtTestCase', 'props expected')


class TestUtil(object):
    DUMMY_REQ_ID = 'req_qsxPhqHyLxcoaM'

    def test_test_apikey(self, mocker):
        mocker.patch('stripe.api_key', 'sk_test_KOWobxXidxNlIx')
        link = util.dashboard_link(self.DUMMY_REQ_ID)
        assert link == \
            'https://dashboard.stripe.com/test/logs/' + self.DUMMY_REQ_ID

    def test_live_apikey(self, mocker):
        mocker.patch('stripe.api_key', 'sk_live_axwITqZSgTUXSN')
        link = util.dashboard_link(self.DUMMY_REQ_ID)
        assert link == \
            'https://dashboard.stripe.com/live/logs/' + self.DUMMY_REQ_ID

    def test_no_apikey(self, mocker):
        mocker.patch('stripe.api_key', None)
        link = util.dashboard_link(self.DUMMY_REQ_ID)
        assert link == \
            'https://dashboard.stripe.com/test/logs/' + self.DUMMY_REQ_ID

    def test_old_apikey(self, mocker):
        mocker.patch('stripe.api_key', 'axwITqZSgTUXSN')
        link = util.dashboard_link(self.DUMMY_REQ_ID)
        assert link == \
            'https://dashboard.stripe.com/test/logs/' + self.DUMMY_REQ_ID

    def log_test_loop(self, test_cases, logging_func, logger_name, mocker):
        for case in test_cases:
            try:
                logger_mock = mocker.patch(logger_name)
                print_mock = mocker.patch(PRINT_FUNC_STRING)
                mocker.patch('stripe.log', case.flag)
                mocker.patch('stripe.util.STRIPE_LOG', case.env)

                logging_func('foo \nbar', y=3)  # function under test

                if case.should_output:
                    print_mock.assert_called_once_with(
                        "message='foo \\nbar' y=3",
                        file=sys.stderr,
                    )
                else:
                    print_mock.assert_not_called()
                logger_mock.assert_called_once_with(
                    "message='foo \\nbar' y=3")
            finally:
                mocker.stopall()

    def test_log_debug(self, mocker):
        # (STRIPE_LOG, stripe.log): should_output?
        test_cases = [
            LogTestCase(env=None, flag=None, should_output=False),
            LogTestCase(env=None, flag='debug', should_output=True),
            LogTestCase(env=None, flag='info', should_output=False),
            LogTestCase(env='debug', flag=None, should_output=True),
            LogTestCase(env='debug', flag='debug', should_output=True),
            LogTestCase(env='debug', flag='info', should_output=False),
            LogTestCase(env='info', flag=None, should_output=False),
            LogTestCase(env='info', flag='debug', should_output=True),
            LogTestCase(env='info', flag='info', should_output=False),
        ]
        self.log_test_loop(
            test_cases,
            logging_func=util.log_debug,
            logger_name='stripe.util.logger.debug',
            mocker=mocker
        )

    def test_log_info(self, mocker):
        # (STRIPE_LOG, stripe.log): should_output?
        test_cases = [
            LogTestCase(env=None, flag=None, should_output=False),
            LogTestCase(env=None, flag='debug', should_output=True),
            LogTestCase(env=None, flag='info', should_output=True),
            LogTestCase(env='debug', flag=None, should_output=True),
            LogTestCase(env='debug', flag='debug', should_output=True),
            LogTestCase(env='debug', flag='info', should_output=True),
            LogTestCase(env='info', flag=None, should_output=True),
            LogTestCase(env='info', flag='debug', should_output=True),
            LogTestCase(env='info', flag='info', should_output=True),
        ]
        self.log_test_loop(
            test_cases,
            logging_func=util.log_info,
            logger_name='stripe.util.logger.info',
            mocker=mocker
        )

    def test_logfmt(self):
        cases = [
            FmtTestCase(props={'foo': 'bar', 'message': 'yes'},
                        expected='foo=bar message=yes'),
            FmtTestCase(props={'foo': 'bar baz'},
                        expected="foo='bar baz'"),
            FmtTestCase(props={'b': 2, 'a': 1, 'c': 3},
                        expected='a=1 b=2 c=3'),
            FmtTestCase(props={'key with space': True},
                        expected="'key with space'=True"),
        ]
        for case in cases:
            result = util.logfmt(case.props)
            assert result == case.expected



from __future__ import absolute_import, division, print_function

import stripe


class TestCreateableAPIResource(object):
    class MyCreatable(stripe.api_resources.abstract.CreateableAPIResource):
        OBJECT_NAME = 'mycreatable'

    def test_create(self, request_mock):
        request_mock.stub_request(
            'post',
            '/v1/mycreatables',
            {
                'object': 'charge',
                'foo': 'bar',
            },
            rheaders={'request-id': 'req_id'}
        )

        res = self.MyCreatable.create()

        request_mock.assert_requested(
            'post',
            '/v1/mycreatables',
            {}
        )
        assert isinstance(res, stripe.Charge)
        assert res.foo == 'bar'

        assert res.last_response is not None
        assert res.last_response.request_id == 'req_id'

    def test_idempotent_create(self, request_mock):
        request_mock.stub_request(
            'post',
            '/v1/mycreatables',
            {
                'object': 'charge',
                'foo': 'bar',
            },
            rheaders={'idempotency-key': 'foo'}
        )

        res = self.MyCreatable.create(idempotency_key='foo')

        request_mock.assert_requested(
            'post',
            '/v1/mycreatables',
            {},
            {'Idempotency-Key': 'foo'}
        )
        assert isinstance(res, stripe.Charge)
        assert res.foo == 'bar'



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

from stripe import six, error


class TestStripeError(object):
    def test_formatting(self):
        err = error.StripeError(u'öre')
        assert six.text_type(err) == u'öre'
        if six.PY2:
            assert str(err) == '\xc3\xb6re'
        else:
            assert str(err) == u'öre'

    def test_formatting_with_request_id(self):
        err = error.StripeError(u'öre', headers={'request-id': '123'})
        assert six.text_type(err) == u'Request 123: öre'
        if six.PY2:
            assert str(err) == 'Request 123: \xc3\xb6re'
        else:
            assert str(err) == u'Request 123: öre'

    def test_formatting_with_none(self):
        err = error.StripeError(None, headers={'request-id': '123'})
        assert six.text_type(err) == u'Request 123: <empty message>'
        if six.PY2:
            assert str(err) == 'Request 123: <empty message>'
        else:
            assert str(err) == 'Request 123: <empty message>'

    def test_formatting_with_message_none_and_request_id_none(self):
        err = error.StripeError(None)
        assert six.text_type(err) == u'<empty message>'
        if six.PY2:
            assert str(err) == '<empty message>'
        else:
            assert str(err) == u'<empty message>'

    def test_repr(self):
        err = error.StripeError(u'öre', headers={'request-id': '123'})
        if six.PY2:
            assert repr(err) == \
                "StripeError(message=u'\\xf6re', http_status=None, " \
                "request_id='123')"
        else:
            assert repr(err) == \
                "StripeError(message='öre', http_status=None, " \
                "request_id='123')"


class TestStripeErrorWithParamCode(object):
    def test_repr(self):
        err = error.CardError(u'öre', param='cparam', code='ccode',
                              http_status=403, headers={'request-id': '123'})
        if six.PY2:
            assert repr(err) == \
                "CardError(message=u'\\xf6re', param='cparam', " \
                "code='ccode', http_status=403, request_id='123')"
        else:
            assert repr(err) == \
                "CardError(message='öre', param='cparam', code='ccode', " \
                "http_status=403, request_id='123')"
